"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfluencePageContentSchema = exports.BaseConfluenceContentSchema = exports.ConfluencePageDetailedV2Schema = exports.ConfluencePageV2Schema = exports.ConfluencePageAncestorSchema = exports.ConfluenceVersionSchema = exports.ConfluencePageBodyViewSchema = exports.ConfluencePageBodyStorageSchema = exports.ConfluenceSpaceDetailedSchema = exports.ConfluenceSpaceSchema = exports.PaginatedResponseSchema = void 0;
// src/services/vendor.atlassian.confluence.types.ts
const zod_1 = require("zod");
// Generic Paginated Response
const PaginatedResponseSchema = (itemSchema) => zod_1.z.object({
    results: zod_1.z.array(itemSchema),
    start: zod_1.z.number().optional(), // v1
    limit: zod_1.z.number().optional(), // v1
    size: zod_1.z.number().optional(), // v1
    _links: zod_1.z
        .object({
        // v1
        next: zod_1.z.string().optional(),
        base: zod_1.z.string().optional(),
        context: zod_1.z.string().optional(),
    })
        .optional(),
    // For v2 pagination (cursor-based)
    next: zod_1.z
        .object({
        // Partial, add more from actual API as needed for v2
        cursor: zod_1.z.string(),
        _links: zod_1.z.object({
            next: zod_1.z.string(),
        }),
    })
        .optional(),
});
exports.PaginatedResponseSchema = PaginatedResponseSchema;
// Confluence Space (for listing results from /api/v2/spaces?keys=...)
exports.ConfluenceSpaceSchema = zod_1.z.object({
    id: zod_1.z.string().describe('ID of the space'), // v2 API uses string IDs for spaces
    key: zod_1.z.string().describe('Key of the space'),
    name: zod_1.z.string().describe('Name of the space'),
    type: zod_1.z.string().optional(),
    status: zod_1.z.string().optional(),
    // Add other relevant fields from the v2 space list item if needed
});
// Confluence Space Detailed (from /api/v2/spaces/{id} or expanded from list)
exports.ConfluenceSpaceDetailedSchema = exports.ConfluenceSpaceSchema.extend({
    description: zod_1.z.any().nullish().optional(), // Allow any type for description or null/undefined
    homepageId: zod_1.z
        .string()
        .nullish()
        .optional()
        .describe('ID of the space homepage (string in v2)'),
    // icon: z.string().optional(), // URL to the space icon
    // Look for description structure in v2, it might be an object like { plain: { value: ..., representation: ... } }
    // For simplicity, keeping as string for now, assuming it might be plain text or needs processing.
});
// Confluence Page Body Content
exports.ConfluencePageBodyStorageSchema = zod_1.z.object({
    value: zod_1.z.string(),
    representation: zod_1.z.literal('storage'),
});
exports.ConfluencePageBodyViewSchema = zod_1.z.object({
    value: zod_1.z.string(),
    representation: zod_1.z.literal('view'),
});
// Confluence Page Version
exports.ConfluenceVersionSchema = zod_1.z.object({
    number: zod_1.z.number(),
    createdAt: zod_1.z.string().datetime().optional(), // ISO 8601 date-time string, used in v2 API
    when: zod_1.z.string().datetime().optional(), // ISO 8601 date-time string, used in v1 API
    message: zod_1.z.string().optional(),
    minorEdit: zod_1.z.boolean().optional(),
    authorId: zod_1.z.string().optional(), // In v2 API
    by: zod_1.z.object({ accountId: zod_1.z.string() }).optional(), // In v1 API, simplified author
    ncsStepVersion: zod_1.z.string().nullish().optional(), // Allow null values
});
// Confluence Page Ancestor (Simplified)
exports.ConfluencePageAncestorSchema = zod_1.z.object({
    id: zod_1.z.string(), // Page IDs are strings in content API, numbers in v2 pages API
    title: zod_1.z.string(),
});
// Confluence Page (for listing results from v2 API)
exports.ConfluencePageV2Schema = zod_1.z.object({
    id: zod_1.z.string().describe('ID of the page'),
    spaceId: zod_1.z.string().describe('ID of the space the page belongs to'),
    status: zod_1.z.string(),
    title: zod_1.z.string(),
    parentId: zod_1.z.string().nullish().optional(),
    position: zod_1.z.number().optional(),
    createdAt: zod_1.z.string().datetime().describe('Creation timestamp of the page'),
    version: zod_1.z
        .object({
        number: zod_1.z.number(),
        message: zod_1.z.string().optional(),
        minorEdit: zod_1.z.boolean().optional(),
        authorId: zod_1.z.string().optional(),
        createdAt: zod_1.z.string().datetime().optional(),
        ncsStepVersion: zod_1.z.string().nullish().optional(), // Allow null values
    })
        .optional(), // Updated to match actual API response
    // _links: z.any().optional(), // For HATEOAS links
});
// Confluence Page Detailed (from v2 API)
exports.ConfluencePageDetailedV2Schema = exports.ConfluencePageV2Schema.extend({
    body: zod_1.z.object({
        storage: exports.ConfluencePageBodyStorageSchema.optional(),
        view: exports.ConfluencePageBodyViewSchema.optional(), // If ever needed
        // raw: ... if ever needed
    }),
    version: exports.ConfluenceVersionSchema, // Full version details
    ancestors: zod_1.z.array(exports.ConfluencePageAncestorSchema).optional(),
    // authors: z.any().optional(),
    // labels: z.any().optional(),
});
// Using a generic for content because v1 content can be page, comment, attachment etc.
exports.BaseConfluenceContentSchema = zod_1.z.object({
    id: zod_1.z.string(),
    type: zod_1.z.string(), // e.g. 'page', 'comment', 'attachment'
    status: zod_1.z.string(),
    title: zod_1.z.string().optional(), // Not present for all content types
});
exports.ConfluencePageContentSchema = exports.BaseConfluenceContentSchema.extend({
    type: zod_1.z.literal('page'),
    title: zod_1.z.string(), // Title is mandatory for pages
    _expandable: zod_1.z
        .object({
        ancestors: zod_1.z.string().optional(),
        body: zod_1.z.string().optional(),
        version: zod_1.z.string().optional(),
    })
        .optional(),
    body: zod_1.z
        .object({
        storage: exports.ConfluencePageBodyStorageSchema.optional(),
        view: exports.ConfluencePageBodyViewSchema.optional(),
    })
        .optional(),
    version: exports.ConfluenceVersionSchema.optional(),
    ancestors: zod_1.z.array(exports.ConfluencePageAncestorSchema).optional(),
});
